#include "bounce_objects.h"
#include "bounce_resources.h"

//--------------------- User code ---------------------//
#include "game_settings.h"
#include "bwi_resources.h"
#include "_bwo_lib.h"

PointF ball_pos;

void Set_Defaults();
void Start_Game();
void Get_Input();
void Animate_Ball();
int Render_Screen();
void SavePrevious();

extern struct rot_xform{
  float x;
  float y;
  float z;
} rotate;

char IsCollision (unsigned int Shape_X, unsigned int Shape_Y,  unsigned int Shape_Width, unsigned int Shape_Height, unsigned int Player_Left, unsigned int Player_Top, unsigned int Player_Width, unsigned int Player_Height);

const animation_address;
unsigned running;
struct ball_object the_ball;

void Start_Game(){
  // Clear the lcd screen for the first time
  if(running == 0){
    ball_pos.X = the_ball.x;
    ball_pos.Y = the_ball.y;

    // Set the pen to 1px and green
    TFT_Set_Pen(GRASS_COLOR, 1);
    // Set the fill brush to a green color
    TFT_Set_Brush(1, GRASS_COLOR, 0, TOP_TO_BOTTOM, GRASS_COLOR, GRASS_COLOR);

    // Clear the soccer field
    TFT_Rectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
  }

  // Set the game is running flag
  running=1;
}

void Set_Defaults() {
  //TRISA = 0xFFFF;        // Initialize PORTA inputs 
  //TRISD = 0xFFFF;        // Initialize PORTD inputs
  //TRISF = 0xFFFF;        // Initialize PORTF inputs
  
  TRISA.B6 = 1; //up
  TRISA.B7 = 1; //down
  TRISD.B10 = 1; //right
  TRISA.B5 = 1; //left
  
  TRISD.B14 = 1; //triangle up
  TRISD.B15 = 1; //x down
  TRISF.B4 = 1; //circle right
  TRISD.B11 = 1; //square left

  TRISF.B5 = 1; //start

  running = 0; 
  frame_counter = 0;
  
  the_ball.frame = 1;

  the_ball.x = SCREEN_WIDTH/2;
  the_ball.y = SCREEN_HEIGHT/2;
  
  the_ball.w = 22;
  the_ball.h = 22;
  
  the_ball.direction_x = 1;
  the_ball.direction_y = -1;
  
  the_ball.x_speed = BALL_X_MAX_SPEED;
  the_ball.y_speed = BALL_Y_MAX_SPEED;
}

// Check for inputs
void Get_Input(){

  // Set the ball x speed
  if((abs(rotate.x)/2) > 10){
    the_ball.x_speed = BALL_X_MAX_SPEED;
  } else {
    the_ball.x_speed = abs(rotate.x)/2;
  }

  // Set the ball y speed
  if((abs(rotate.y)/2) > 10){
    the_ball.y_speed = BALL_Y_MAX_SPEED;
  } else {
    the_ball.y_speed = abs(rotate.y)/2;
  }
  // Accelerometer Horizontal Tipping
  if(rotate.x > 5){
    the_ball.direction_x = 1;
  } else if (rotate.x < -5){
    the_ball.direction_x = -1;
  } else {
    the_ball.direction_x = 0;
    the_ball.x_speed = BALL_X_MIN_SPEED;
  }
  
  // Accelerometer Vertical Tipping
  if(rotate.y > 5){
    the_ball.direction_y = 1;
  } else if (rotate.y < -5){
    the_ball.direction_y = -1;
  } else {
    the_ball.direction_y = 0;
    the_ball.y_speed = BALL_Y_MIN_SPEED;
  }
}


void Animate_Ball(){
  // The the_ball hit the left side of the screen
  if(the_ball.x < (5+the_ball.x_speed)){
    the_ball.direction_x = -1;
    ~the_ball.direction_y;
    the_ball.x_speed = BALL_X_MIN_SPEED+1;
  }
  
  // The the_ball hit the right side of the screen
  else if(the_ball.x > ((SCREEN_WIDTH)-the_ball.w) ){
    the_ball.direction_x = 1;
    ~the_ball.direction_y;
    the_ball.x_speed = BALL_X_MIN_SPEED+1;
  }

  // The the_ball hit the top of the screen
  if(the_ball.y < (5+the_ball.y_speed)){
    the_ball.direction_y = -1;
    ~the_ball.direction_x;
    the_ball.y_speed = BALL_Y_MIN_SPEED+1;
  }
  
  // The the_ball hit the bottom of the screen
  else if(the_ball.y > ((SCREEN_HEIGHT)-the_ball.h) ){
    the_ball.direction_y = 1;
    ~the_ball.direction_x;
    the_ball.y_speed = BALL_Y_MIN_SPEED+1;
  }

  // Reset the ball speed on a wall bounce
  if( (the_ball.prev_direction_x != the_ball.direction_x) ||
      (the_ball.prev_direction_y != the_ball.direction_y) ){
    //the_ball.x_speed = BALL_X_MIN_SPEED;
    //the_ball.y_speed = BALL_Y_MIN_SPEED;
  }

  // Slow the ball sprite down over time
  if((frame_counter % 8) == 1){
    // Check if the ball is going the min x speed
    if(the_ball.x_speed > BALL_X_MIN_SPEED){
      the_ball.x_speed--;
    }
    // Check if the ball is going the min y speed
    if(the_ball.y_speed > BALL_Y_MIN_SPEED){
      the_ball.y_speed--;
    }
  }

  // Keep the ball X speed above the minimum
  if(the_ball.x_speed < BALL_X_MIN_SPEED){
    the_ball.x_speed = BALL_X_MIN_SPEED;
  }

  // Keep the ball Y speed above the minimum
  if(the_ball.y_speed < BALL_Y_MIN_SPEED){
    the_ball.y_speed = BALL_Y_MIN_SPEED;
  }
  
  // Animate the ball
  the_ball.x -= the_ball.direction_x * the_ball.x_speed;
  the_ball.y -= the_ball.direction_y * the_ball.y_speed;

  // Increment ball sprite frame
  if( (the_ball.x_speed >=5) || (the_ball.y_speed >=5) ) {
    if(the_ball.direction_x == 1) {
      the_ball.frame++;
    }
    else {
      the_ball.frame--;
    }
  }
  else if( (the_ball.x_speed >=3) || (the_ball.y_speed >=3) ) {
    if( (frame_counter % 2 ) == 0){
      if(the_ball.direction_x == 1) {
        the_ball.frame++;
      } else {
        the_ball.frame--;
      }
    }
  } else if ((the_ball.x_speed >=1) || (the_ball.y_speed >=1) )  {
    if( (frame_counter % 4 ) == 0){
      if(the_ball.direction_x == 1) {
        the_ball.frame++;
      } else {
        the_ball.frame--;
      }
    }
  }
  
  // Keep the frame range valid
  if (the_ball.frame > 15){
    the_ball.frame = 1;
  } else if (the_ball.frame <= 0){
    the_ball.frame = 15;
  }
  
  ball_pos.X = the_ball.x;
  ball_pos.Y = the_ball.y;
  
  switch (the_ball.frame) {
  case 1:
    image_bwi(ball_pos, ball_01_bwi);
    break;
    
  case 2:
    image_bwi(ball_pos, ball_02_bwi);
    break;
    
  case 3:
    image_bwi(ball_pos, ball_03_bwi);
    break;
    
  case 4:
    image_bwi(ball_pos, ball_04_bwi);
    break;
    
  case 5:
    image_bwi(ball_pos, ball_05_bwi);
    break;
    
  case 6:
    image_bwi(ball_pos, ball_06_bwi);
    break;
    
  case 7:
    image_bwi(ball_pos, ball_07_bwi);
    break;
    
  case 8:
    image_bwi(ball_pos, ball_08_bwi);
    break;
    
  case 9:
    image_bwi(ball_pos, ball_09_bwi);
    break;
    
  case 10:
    image_bwi(ball_pos, ball_10_bwi);
    break;
    
  case 11:
    image_bwi(ball_pos, ball_11_bwi);
    break;
    
  case 12:
    image_bwi(ball_pos, ball_12_bwi);
    break;
    
  case 13:
    image_bwi(ball_pos, ball_13_bwi);
    break;
    
  case 14:
    image_bwi(ball_pos, ball_14_bwi);
    break;
    
  case 15:
    image_bwi(ball_pos, ball_15_bwi);
    break;

  default:
    image_bwi(ball_pos, ball_01_bwi);
    break;
  }
}


// Check for a sprite to Player collision
char IsCollision (unsigned int Shape_X, unsigned int Shape_Y,  unsigned int Shape_Width, unsigned int Shape_Height, unsigned int Player_Left, unsigned int Player_Top, unsigned int Player_Width, unsigned int Player_Height) {
  
  // Check for Player collisions
  // The right side of Shape is greater than left side of Player     and        left side of Shape is smaller than right side of Player
  if( ((Shape_X + Shape_Width) >= Player_Left )               &&              ((Shape_X) <= (Player_Left + Player_Width-1)) &&

      // Check for Player top and bottom collisions
      //The bottom side of Shape is greater than top of Player         and            top of Shape is smaller then bottom of Player
      ((Shape_Y + Shape_Height) >= Player_Top )                   &&               ((Shape_Y) <= (Player_Top + Player_Height-1))  ) {
    // Debug printing
    //printf("Collision detected\n");
    
    // A collision has been detected
    return 1;
  } else {
    // Default return value
    // No collisions have been detected
    return 0;
  }
}

int Render_Screen() {

  if (running != 1)
  return 0;

  // Slow down the animation
  Delay_ms(30);

  frame_counter++;

  // Set the pen to 1px and green
  TFT_Set_Pen(GRASS_COLOR, 1);

  // Set the fill brush to a green color
  TFT_Set_Brush(1, GRASS_COLOR, 0, TOP_TO_BOTTOM, GRASS_COLOR, GRASS_COLOR);
  
  if(the_ball.y < 0)
  the_ball.y = SCREEN_HEIGHT/2;
  
  if(the_ball.x < 0)
  the_ball.x = SCREEN_WIDTH/2;
  
  if(the_ball.prev_y < 0)
  the_ball.prev_y = SCREEN_HEIGHT/2;

  if(the_ball.prev_x < 0)
  the_ball.prev_x = SCREEN_WIDTH/2;
  
  // Keep objects onscreen

  // Clear the old soccer ball
  TFT_Rectangle(the_ball.x, the_ball.y, the_ball.x+the_ball.w, the_ball.y+the_ball.h);

  Animate_Ball();
  
  SavePrevious();
  
  return 1;
}

// Save the settings from the previous frame
void SavePrevious(){
  // Save the old the_ball position
  the_ball.prev_x = the_ball.x;
  the_ball.prev_y = the_ball.y;

  // Save the old the_ball direction
  the_ball.prev_direction_x = the_ball.direction_x;
  the_ball.prev_direction_y = the_ball.direction_y;
}

//----------------- End of User code ------------------//